#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <math.h>
#include <string.h>
#include <time.h>
#include <SDL.h>

#include "fps.h"

void SetPalette(SDL_Surface *screen);
void DrawSpiral(SDL_Surface *screen);
void RotatePalette(SDL_Surface *screen);
int event_test(SDL_Surface *screen);

int wid=640;
int hgt=480;

int quit=0;
int pattern;
int speed=1;
SDL_Color p[256];

//PALETTE p;

SDL_Surface * init_video(int screen_wid, int screen_hgt);

int main(int argc, char **argv)
{
   SDL_Surface *screen;
   int errcode=0;

   srand(time(NULL));
   pattern = rand() % 7;

   if(argc>1) pattern = atoi(argv[1]);

   screen = init_video(wid, hgt);
   //set_gfx_mode(GFX_AUTODETECT, xres, yres, 0, 0);

   // Draw Stuff!
   SetPalette(screen);
   DrawSpiral(screen);
   FpsStart();
   RotatePalette(screen);
   FpsEnd();

   //set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);

   printf("Spiral psychadelic thing, V0.1\n");
   printf("Copyright (C) 2002 Scott Scriven\n");
   printf("\n\n\nIf this text isn't moving, you didn't watch for long enough...\n");
   ShowFps();
   //printf("Fps: %f\n", Fps());

   return errcode;
}

//////////////////////////////////////////////////
//
SDL_Surface * init_video(int screen_wid, int screen_hgt)
{
   SDL_Surface *screen = NULL;
   int color_depth=8;

   // Startup SDL
   if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
      fprintf(stderr, "Unable to init SDL: %s\n", SDL_GetError());
      exit(1);
   }
   atexit(SDL_Quit);

   // Set a video mode
   screen = SDL_SetVideoMode(screen_wid, screen_hgt, color_depth,
			     SDL_HWPALETTE|SDL_FULLSCREEN);
   if ( screen == NULL ) {
      fprintf(stderr, "Unable to set %ix%ix%i video: %s\n",
	      screen_wid, screen_hgt, color_depth, SDL_GetError());
      exit(1);
   }

   return screen;
}

void SetPalette(SDL_Surface *screen)
{
  int i;


/*
  for(i=0; i<8; i++)
  {
    c.r = i*8;
    c.g = i*8;
    c.b = 0;
    set_color(i, &c);
  }
  for(i=8; i<16; i++)
  {
    c.r = (16-i)*8;
    c.g = (16-i)*8;
    c.b = 0;
    set_color(i, &c);
  }
*/

  for(i=0; i<256; i++)
  {
    p[i].r = 127 * ((i & 8)>0) + (2* ((i-1)&8))  + (2* ((i+1)&8));
    p[i].g = 127 * ((i & 8)>0) + (2* ((i-1)&8))  + (2* ((i+1)&8));
    p[i].b = 0;
  }

  SDL_SetPalette(screen, SDL_LOGPAL|SDL_PHYSPAL, p, 0, 256);
/*
  for(i=0; i<128; i++)
  {
    c.r = i/2;
    c.g = i/2;
    c.b = 0;
    set_color(i, &c);
  }
  for(i=128; i<256; i++)
  {
    c.r = (255-i)/2;
    c.g = (255-i)/2;
    c.b = 0;
    set_color(i, &c);
  }
*/
//  get_palette(p);
}

void putpixel(SDL_Surface *surface, int x, int y, Uint8 pixel)
{
   //int bpp = surface->format->BytesPerPixel;
    /* Here p is the address to the pixel we want to set */
    Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x;
    *p = pixel;
}

void DrawSpiral(SDL_Surface *screen)
{
  int x,y, sx,sy;
  char c;
  double angle, radius;

//  pattern = 6;

  for(sy=0; sy<hgt; sy++)
  {
    y = sy - (hgt/2);
    for(sx=0; sx<wid; sx++)
    {
      x = sx - (wid/2);
      if(x != 0  &&  y != 0)
        angle = atan2(x,y);
      radius = sqrt((x*x) + (y*y));

      switch(pattern)
      {
        case 0:
          c = 256 * (angle / (3.1415*2))
            + (radius*2 * (-(!((int)radius&32))));
          break;
        case 1:
          c = 256 * (angle / (3.1415*2));
          if((int)radius&32) c += -(radius*2);
          else c+= radius*2;
          break;
        case 2:
          c = 256 * (angle / (3.1415*2)) + radius;
          break;
        case 3:
          c = 256 * (angle / (3.1415*2)) - radius;
          break;
        case 4:
          c = radius;
          break;
        case 5:
          c = -radius;
          break;
        case 6:
          c = 256 * (angle / (3.1415*2));
          break;
      }
      putpixel(screen, sx, sy, c);
    }
  }

}

void RotatePalette(SDL_Surface *screen)
{
  int i;
  SDL_Color c;

  while(! quit)
  {
     event_test(screen);
     c.r = p[0].r;
     c.g = p[0].g;
     c.b = p[0].b;
     for(i=0; i<255; i++)
     {
	p[i].r = p[i+1].r;
	p[i].g = p[i+1].g;
	p[i].b = p[i+1].b;
     }
     p[255].r = c.r;
     p[255].g = c.g;
     p[255].b = c.b;
     SDL_SetPalette(screen, SDL_LOGPAL|SDL_PHYSPAL, p, 0, 256);
     frames ++;
     if(speed) usleep(speed);

    /*
    if(kbhit())
    {
      key = getch();
      switch(key)
      {
        case '-': speed++; break;
        case '+': if(speed>0) speed--; break;
        case ' ': pattern = rand() % 7;  DrawSpiral();  break;
      }
    }
    */
  }
}


//////////////////////////////////////////////////
//
int event_test(SDL_Surface *screen)
{
   SDL_Event event;
   Uint8 *keys;

   while ( SDL_PollEvent(&event) ) {
      switch (event.type) {
	 case SDL_KEYDOWN:
	    //printf("The %s key was pressed!\n",
	    //	   SDL_GetKeyName(event.key.keysym.sym));
	    keys = SDL_GetKeyState(NULL);
	    if(keys[SDLK_ESCAPE] == SDL_PRESSED)
	    {
	       quit = 1;
	    }
	    else if(keys[SDLK_SPACE] == SDL_PRESSED)
	    {
	       pattern = rand() % 7;  DrawSpiral(screen);  break;
	    }
	    else if(keys[SDLK_KP_PLUS] == SDL_PRESSED)
	    {
	       printf("slower...\n");
	       speed++;
	    }
	    else if(keys[SDLK_KP_MINUS] == SDL_PRESSED)
	    {
	       printf("faster...\n");
	       if(speed > 0)
		  speed--;
	    }
	    break;
	 case SDL_MOUSEMOTION:
	    //printf("Mouse moved by %d,%d to (%d,%d)\n", 
	    //	   event.motion.xrel, event.motion.yrel,
	    //	   event.motion.x, event.motion.y);
	    break;
	 case SDL_MOUSEBUTTONDOWN:
	    printf("Mouse button %d pressed at (%d,%d)\n",
		   event.button.button, event.button.x, event.button.y);
	    break;
	 case SDL_QUIT:
	    exit(0);
      }
   }
   
   return 0;
}
